from typing import Any

from django.conf import settings
from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.http import HttpRequest
from modeltranslation.admin import TranslationAdmin

from django_features.custom_fields import models
from django_features.custom_fields.models import CustomFieldBaseModel
from django_features.custom_fields.models import CustomFieldTypeBaseModel


class CustomFieldBaseAdmin(TranslationAdmin):
    def has_module_permission(self, request: HttpRequest) -> bool:
        return settings.CUSTOM_FIELDS_FEATURE

    def get_form(self, request: HttpRequest, obj: Any = None, **kwargs: Any) -> Any:
        form = super().get_form(request, obj, **kwargs)
        form.base_fields["content_type"].queryset = ContentType.objects.filter(
            id__in=[
                content_type.id
                for content_type in ContentType.objects.all()
                if content_type.model_class() is not None
                and issubclass(content_type.model_class(), CustomFieldBaseModel)
            ]
        )
        form.base_fields["type_content_type"].queryset = ContentType.objects.filter(
            id__in=[
                content_type.id
                for content_type in ContentType.objects.all()
                if content_type.model_class() is not None
                and issubclass(content_type.model_class(), CustomFieldTypeBaseModel)
            ]
        )
        return form

    def get_readonly_fields(self, request: HttpRequest, obj: Any = None) -> list[str]:
        if obj:
            return [*self.readonly_fields, "field_type"]
        return self.readonly_fields


@admin.register(models.CustomField)
class CustomFieldAdmin(CustomFieldBaseAdmin):
    list_display = ["id", "identifier", "__str__", "field_type"]
    list_display_links = (
        "id",
        "identifier",
        "__str__",
    )
    list_filter = ("choice_field", "editable", "field_type", "content_type")
    search_fields = ("label", "identifier")


@admin.register(models.CustomValue)
class ValueAdmin(TranslationAdmin):
    list_display = ["id", "__str__"]
    search_fields = ("label", "value", "field__label", "field__identifier")



from django import forms
from django.forms import widgets


class SystemMessageAdminForm(forms.ModelForm):
    class Meta:
        fields = "__all__"
        widgets = {
            "background_color": widgets.TextInput(attrs={"type": "color"}),
            "text_color": widgets.TextInput(attrs={"type": "color"}),
        }



import json
from typing import Any

from django.forms import fields

from django_features.validations import MappingValidationMixin


class PrettyJSONEncoder(json.JSONEncoder):
    def __init__(self, *args: Any, indent: int, sort_keys: bool, **kwargs: Any) -> None:
        super().__init__(*args, indent=2, **kwargs)


class PrettyJSONField(fields.JSONField):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        kwargs["encoder"] = PrettyJSONEncoder
        super().__init__(*args, **kwargs)


class ModelFieldMapping(MappingValidationMixin, PrettyJSONField):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)

    def validate(self, value: Any) -> None:
        super().validate(value)
        if value is None:
            return
        self.validate_models_field_mapping(value)



from typing import Any

from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models import QuerySet
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from rest_framework.exceptions import ValidationError


class UUIDRelatedField(serializers.RelatedField):
    """
    A relation field which expects an uid to be present on the related model for lookup and representation.
    """

    default_uuid_field_name = "uid"

    default_error_messages = {
        "does_not_exist": _("Objekt existiert nicht."),
        "incorrect_type": _("Inkorrekter Typ."),
    }

    def __init__(
        self,
        field: models.Field | None = None,
        queryset: QuerySet | None = None,
        uuid_field_name: str | None = None,
        **kwargs: Any,
    ) -> None:
        self.field = field
        self.queryset = queryset
        self.uuid_field_name = uuid_field_name or self.default_uuid_field_name
        super().__init__(**kwargs)

    def get_field(self) -> models.Field:
        if self.field:
            field = self.field
        elif self.queryset:
            field = self.queryset.model._meta.get_field(self.field_name)
        elif self.parent.Meta and self.parent.Meta.model:
            field = self.parent.Meta.model._meta.get_field(self.field_name)
        else:
            raise ValidationError(
                f"No field or queryset defined for field {self.field_name} "
                f"and parent class {self.parent.__class__} has no attribute Meta."
            )
        if not field.is_relation:
            raise ValidationError(f"Field {self.field_name} is not a relation field.")
        return field

    def get_queryset(self) -> QuerySet:
        return self.queryset or self.get_field().related_model.objects

    def to_representation(self, related_obj: models.Model) -> None | str:
        """
        Get the UID of the related object; format it as UID.
        """
        if not related_obj:
            return None
        related_uid = getattr(related_obj, self.uuid_field_name)
        if related_uid is None:
            raise AttributeError(
                f"No field named 'UID' defined for relation to {self.field_name} on {related_obj}"
            )
        return serializers.UUIDField().to_representation(related_uid)

    def to_internal_value(self, data: str) -> models.Model | None:
        """
        Assume that the model is defined on the serializer using this field.
        """
        if not data and not self.required:
            return None
        uid = serializers.UUIDField().to_internal_value(data)
        try:
            return self.get_queryset().get(**{self.uuid_field_name: uid})
        except ObjectDoesNotExist as e:
            raise ValidationError(
                f"Object {self.get_field().related_model} with uid {uid} does not exist: {e}"
            )
        except (TypeError, ValueError) as e:
            raise ValidationError(
                f"The data {uid} for model {self.get_field().related_model} has an incorrect type {type(data).__name__}: {e}"
            )



# Generated by Django 4.2.23 on 2025-09-03 03:13

import django.db.models.deletion
import django_extensions.db.fields
from django.conf import settings
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="SystemMessageType",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name="modified"
                    ),
                ),
                ("name", models.CharField(max_length=32, verbose_name="Name")),
                (
                    "icon",
                    models.CharField(
                        blank=True,
                        help_text="Icon Name von https://mdisearch.com/ ohne mdi-Prefix mit Bindestrich getrennt",
                        max_length=255,
                        null=True,
                        verbose_name="Icon",
                    ),
                ),
            ],
            options={
                "verbose_name": "Systemmeldungstyp",
                "verbose_name_plural": "Systemmeldungstypen",
                "ordering": ("name",),
            },
        ),
        migrations.CreateModel(
            name="SystemMessage",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name="modified"
                    ),
                ),
                (
                    "background_color",
                    models.CharField(
                        default="#0000FF", max_length=7, verbose_name="Hintergrundfarbe"
                    ),
                ),
                ("begin", models.DateTimeField(verbose_name="Beginn")),
                (
                    "end",
                    models.DateTimeField(blank=True, null=True, verbose_name="Ende"),
                ),
                (
                    "order",
                    models.PositiveIntegerField(default=0, verbose_name="Reihenfolge"),
                ),
                ("text", models.TextField(blank=True, verbose_name="Nachricht")),
                (
                    "text_color",
                    models.CharField(
                        default="#000000", max_length=7, verbose_name="Textfarbe"
                    ),
                ),
                ("title", models.CharField(max_length=32, verbose_name="Titel")),
                (
                    "dismissed_users",
                    models.ManyToManyField(
                        related_name="dismissed_system_messages",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="system_messages",
                        to="system_message.systemmessagetype",
                        verbose_name="Typ",
                    ),
                ),
            ],
            options={
                "verbose_name": "Systemmeldung",
                "verbose_name_plural": "Systemmeldungen",
                "ordering": ("order", "end", "begin", "title"),
            },
        ),
    ]



# Generated by Django 4.2.23 on 2025-10-22 11:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("custom_fields", "0003_add_type_filter_to_fields"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="customfield",
            name="multiple_choice",
        ),
    ]



# Generated by Django 4.2.23 on 2025-10-22 12:04

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ("custom_fields", "0004_remove_mutiple_choice_field"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="customvalue",
            name="text",
        ),
        migrations.RemoveField(
            model_name="customvalue",
            name="text_de",
        ),
        migrations.RemoveField(
            model_name="customvalue",
            name="text_en",
        ),
        migrations.RemoveField(
            model_name="customvalue",
            name="text_fr",
        ),
        migrations.AddField(
            model_name="customvalue",
            name="label",
            field=models.CharField(blank=True, null=True, verbose_name="Label"),
        ),
        migrations.AddField(
            model_name="customvalue",
            name="label_de",
            field=models.CharField(blank=True, null=True, verbose_name="Label"),
        ),
        migrations.AddField(
            model_name="customvalue",
            name="label_en",
            field=models.CharField(blank=True, null=True, verbose_name="Label"),
        ),
        migrations.AddField(
            model_name="customvalue",
            name="label_fr",
            field=models.CharField(blank=True, null=True, verbose_name="Label"),
        ),
    ]



from django.conf import settings
from rest_framework.pagination import PageNumberPagination


class PageNumberPaginator(PageNumberPagination):
    page_query_param = settings.PAGE_QUERY_PARAM
    page_size_query_param = settings.PAGE_SIZE_QUERY_PARAM



from constance import config
from django.contrib import admin
from django.http import HttpRequest
from django.utils.safestring import mark_safe
from django.utils.safestring import SafeString

from django_features.system_message import forms
from django_features.system_message import models


class SystemMessageBaseAdmin(admin.ModelAdmin):
    def has_module_permission(self, request: HttpRequest) -> bool:
        return config.ENABLE_SYSTEM_MESSAGE


@admin.register(models.SystemMessageType)
class SystemInfoTypeAdmin(SystemMessageBaseAdmin):
    list_display = (
        "id",
        "name",
        "icon",
    )
    list_display_links = ["id", "name"]
    search_fields = [
        "id",
        "name",
        "icon",
    ]


@admin.register(models.SystemMessage)
class SystemInfoAdmin(SystemMessageBaseAdmin):
    form = forms.SystemMessageAdminForm
    list_display = (
        "id",
        "__str__",
        "type",
        "get_background_preview",
        "background_color",
        "get_text_preview",
        "text_color",
    )
    list_display_links = ["id", "__str__"]
    list_filter = ("type",)
    search_fields = [
        "background_color",
        "text",
        "id",
        "text_color",
        "title",
        "type",
    ]

    def get_background_preview(self, obj: models.SystemMessage) -> SafeString:
        return mark_safe(
            f"<div style='background-color: {obj.background_color}; width: 20px;'>&nbsp;</div>"
        )

    def get_text_preview(self, obj: models.SystemMessage) -> SafeString:
        return mark_safe(
            f"<div style='background-color: {obj.text_color}; width: 20px;'>&nbsp;</div>"
        )



# Generated by Django 4.2.23 on 2025-09-04 07:42

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ("system_message", "0002_create_default_system_message_types"),
    ]

    operations = [
        migrations.AlterField(
            model_name="systemmessage",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="systemmessagetype",
            name="icon",
            field=models.CharField(
                blank=True,
                help_text="Icon Name von mdi ohne mdi-Prefix mit Bindestrich getrennt",
                max_length=255,
                null=True,
                verbose_name="Icon",
            ),
        ),
        migrations.AlterField(
            model_name="systemmessagetype",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



from modeltranslation.decorators import register
from modeltranslation.translator import TranslationOptions

from django_features.custom_fields import models


@register(models.CustomField)
class CustomFieldTranslationOptions(TranslationOptions):
    fields = ("label",)


@register(models.CustomValue)
class CustomChoiceTranslationOptions(TranslationOptions):
    fields = ("label",)



# Generated by Django 4.2.23 on 2025-10-09 05:44

import django.db.models.deletion
import django_extensions.db.fields
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomField",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name="modified"
                    ),
                ),
                (
                    "allow_blank",
                    models.BooleanField(
                        default=True, verbose_name="Leeren String erlauben"
                    ),
                ),
                (
                    "allow_null",
                    models.BooleanField(
                        default=True, verbose_name="Leere Werte erlauben"
                    ),
                ),
                (
                    "choice_field",
                    models.BooleanField(default=False, verbose_name="Auswahlfeld"),
                ),
                (
                    "default",
                    models.JSONField(
                        blank=True, null=True, verbose_name="Standardwert"
                    ),
                ),
                (
                    "editable",
                    models.BooleanField(default=True, verbose_name="Editierbar"),
                ),
                (
                    "external_key",
                    models.CharField(
                        blank=True, null=True, verbose_name="Externer Key"
                    ),
                ),
                (
                    "field_type",
                    models.CharField(
                        choices=[
                            ("CHAR", "Text (einzeilig)"),
                            ("TEXT", "Text (mehrzeilig)"),
                            ("DATE", "Datum"),
                            ("DATETIME", "Datum und Zeit"),
                            ("INTEGER", "Zahl (Ganzzahl)"),
                            ("BOOLEAN", "Checkbox"),
                        ],
                        verbose_name="Feldtyp",
                    ),
                ),
                (
                    "hidden",
                    models.BooleanField(default=False, verbose_name="Ausblenden"),
                ),
                ("identifier", models.SlugField(max_length=64, unique=True)),
                ("label", models.CharField(verbose_name="Name")),
                ("multiple", models.BooleanField(default=False, verbose_name="Liste")),
                (
                    "multiple_choice",
                    models.BooleanField(
                        default=False, verbose_name="Mehrfachauswahlfeld"
                    ),
                ),
                (
                    "order",
                    models.PositiveSmallIntegerField(
                        default=0, verbose_name="Reihenfolge"
                    ),
                ),
                (
                    "required",
                    models.BooleanField(default=False, verbose_name="Erforderlich"),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
            options={
                "verbose_name": "Benutzerdefiniertes Feld",
                "verbose_name_plural": "Benutzerdefinierte Felder",
                "ordering": ["order", "created"],
            },
        ),
        migrations.CreateModel(
            name="CustomValue",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name="modified"
                    ),
                ),
                (
                    "order",
                    models.PositiveIntegerField(default=0, verbose_name="Reihenfolge"),
                ),
                ("text", models.CharField(blank=True, null=True, verbose_name="Text")),
                ("value", models.JSONField(blank=True, null=True, verbose_name="Wert")),
                (
                    "field",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="values",
                        to="custom_fields.customfield",
                        verbose_name="Feld",
                    ),
                ),
            ],
            options={
                "verbose_name": "Benutzerdefinierter Wert",
                "verbose_name_plural": "Benutzerdefinierte Werte",
                "ordering": ["order", "created"],
            },
        ),
    ]



from typing import Any

from django.conf import settings
from django.contrib.contenttypes.fields import GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import FieldDoesNotExist
from django.core.exceptions import ValidationError
from django.db.models import Model
from django.db.models.fields.related import RelatedField
from rest_framework.utils.model_meta import get_field_info

from django_features.custom_fields.models import CustomField


class MappingValidationMixin:
    _special_fields = ["unique_choice_field"]

    def __init__(
        self,
        allow_many_to_many: bool = True,
        allow_relations: bool = True,
        relation_separator: str = ".",
        validate_custom_fields: bool = True,
        validate_required: bool = True,
        validate_key: bool = False,
        validate_value: bool = True,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        self.allow_many_to_many = allow_many_to_many
        self.allow_relations = allow_relations
        self.relation_separator = relation_separator
        self.validate_custom_fields = validate_custom_fields
        self.validate_required = validate_required
        self.validate_key = validate_key
        self.validate_value = validate_value
        super().__init__(*args, **kwargs)

    def validate_model(self, model_path: str) -> type[Model]:
        split_model = model_path.split(".")
        if len(split_model) == 1:
            content_types = ContentType.objects.filter(model=model_path)
        elif len(split_model) == 2:
            content_types = ContentType.objects.filter(
                app_label=split_model[0], model=split_model[1]
            )
        else:
            raise ValueError(f"Invalid model '{model_path}'.")

        if not content_types.exists():
            raise ValidationError(f"Model '{model_path}' does not exist.")
        if content_types.count() > 1:
            raise ValidationError(f"Multiple models found for '{model_path}'.")

        content_type = content_types.first()
        model_class = content_type.model_class()
        if not model_class:
            raise ValidationError(
                f"No model found for content type '{content_type}' with model {model_path}."
            )
        return model_class

    def valid_custom_fields(self, model: type[Model]) -> list:
        if not self.validate_custom_fields or not settings.CUSTOM_FIELDS_FEATURE:
            return []

        content_type = ContentType.objects.get_for_model(model)
        return list(
            CustomField.objects.filter(content_type=content_type)
            .values_list("identifier", flat=True)
            .distinct()
        )

    def validate_field(self, field_path: str, model: type[Model] | Any) -> None:
        split = field_path.split(self.relation_separator)
        field = split[0]

        if field in self.valid_custom_fields(model):
            return

        try:
            field = model._meta.get_field(field)
        except FieldDoesNotExist:
            raise ValidationError(f"Invalid field '{field}' for model {model}.")

        if field.is_relation and not self.allow_relations:
            raise ValidationError(
                f"Field '{field}' is a relation and cannot be assigned."
            )

        if field.many_to_many and not self.allow_many_to_many:
            raise ValidationError(
                f"Field '{field}' is a many-to-many relation and cannot be assigned."
            )

        if len(split) > 1 and isinstance(field, RelatedField):
            if field.many_to_many:
                raise ValidationError(
                    f"Field '{field}' is a many-to-many relation and cannot be nested."
                )
            if isinstance(field, GenericRelation):
                raise ValidationError(
                    f"Field '{field}' is a generic relation and cannot be nested."
                )
            self.validate_field(
                self.relation_separator.join(split[1:]), field.related_model
            )

    def validate_required_fields(
        self, model: type[Model], field_mapping: dict[str, Any]
    ) -> None:
        if not self.validate_required:
            return

        info = get_field_info(model)
        for name, field in info.fields.items():
            if (
                field.null is False
                and (
                    name not in field_mapping.values()
                    and self.validate_value
                    or name not in field_mapping.keys()
                    and self.validate_key
                )
                and (not hasattr(field, "auto_now_add") or field.auto_now_add is False)
                and (not hasattr(field, "auto_now") or field.auto_now is False)
            ):
                raise ValidationError(
                    f"Required field '{name}' not found in field mapping."
                )

    def validate_model_field_mapping(
        self, model: type[Model], field_mapping: dict[str, Any]
    ) -> None:
        if not isinstance(field_mapping, dict):
            raise ValidationError(
                "The value must be a dictionary with field names as keys and values."
            )
        self.validate_required_fields(model, field_mapping)
        for key, value in field_mapping.items():
            if self.validate_key:
                self.validate_field(key, model)
            if self.validate_value:
                self.validate_field(value, model)

    def validate_models_field_mapping(
        self, models_field_mapping: dict[str, Any]
    ) -> None:
        for model_path, field_mapping in models_field_mapping.items():
            if model_path in self._special_fields:
                continue
            model = self.validate_model(model_path)
            self.validate_model_field_mapping(model, field_mapping)



from configurations import Configuration
from configurations import values


class BaseConfiguration(Configuration):
    @property
    def INSTALLED_APPS(self) -> list[str]:
        return []

    PAGE_QUERY_PARAM = values.Value("page")
    PAGE_SIZE_QUERY_PARAM = values.Value("page_size")

    CUSTOM_FIELD_APP = values.Value("django_features.custom_fields")

    @property
    def CUSTOM_FIELDS_FEATURE(self) -> bool:
        return self.CUSTOM_FIELD_APP in self.INSTALLED_APPS

    @property
    def CONSTANCE_ADDITIONAL_FIELDS(self) -> dict:
        return {
            "json": [
                "django_features.settings.fields.PrettyJSONField",
                {"required": False},
            ],
            "model_field_mapping": [
                "django_features.settings.fields.ModelFieldMapping",
                {"required": False},
            ],
        }



# Generated by Django 4.2.23 on 2025-10-03 09:29

from django.conf import settings
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("system_message", "0003_alter_id_fields_with_big_auto_field"),
    ]

    operations = [
        migrations.AlterField(
            model_name="systemmessage",
            name="dismissed_users",
            field=models.ManyToManyField(
                blank=True,
                related_name="dismissed_system_messages",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel


class SystemMessageType(TimeStampedModel):
    name = models.CharField(verbose_name=_("Name"), max_length=32)
    icon = models.CharField(
        verbose_name=_("Icon"),
        help_text=_("Icon Name von mdi ohne mdi-Prefix mit Bindestrich getrennt"),
        max_length=255,
        blank=True,
        null=True,
    )

    class Meta:
        verbose_name = _("Systemmeldungstyp")
        verbose_name_plural = _("Systemmeldungstypen")
        ordering = ("name",)

    def __str__(self) -> str:
        return f"{self.name}"


class SystemMessage(TimeStampedModel):
    background_color = models.CharField(
        verbose_name=_("Hintergrundfarbe"), default="#0000FF", max_length=7
    )
    begin = models.DateTimeField(verbose_name=_("Beginn"))
    end = models.DateTimeField(verbose_name=_("Ende"), blank=True, null=True)
    order = models.PositiveIntegerField(_("Reihenfolge"), default=0)
    text = models.TextField(verbose_name=_("Nachricht"), blank=True)
    text_color = models.CharField(
        verbose_name=_("Textfarbe"), default="#000000", max_length=7
    )
    title = models.CharField(verbose_name=_("Titel"), max_length=32)
    type = models.ForeignKey(
        SystemMessageType,
        on_delete=models.PROTECT,
        related_name="system_messages",
        verbose_name=_("Typ"),
    )
    dismissed_users = models.ManyToManyField(
        get_user_model(),
        related_name="dismissed_system_messages",
        blank=True,
    )

    class Meta:
        verbose_name = _("Systemmeldung")
        verbose_name_plural = _("Systemmeldungen")
        ordering = ("order", "end", "begin", "title")

    def __str__(self) -> str:
        return f"{self.title}"



from rest_framework.routers import DefaultRouter

from django_features.custom_fields.viewsets import CustomFieldViewSet


custom_field_router = DefaultRouter(trailing_slash=False)
custom_field_router.register(r"custom_field", CustomFieldViewSet)



from django.db.models import QuerySet
from rest_framework.viewsets import ReadOnlyModelViewSet

from django_features.custom_fields import models
from django_features.custom_fields import serializers


class CustomFieldViewSet(ReadOnlyModelViewSet):
    queryset = models.CustomField.objects.all()
    serializer_class = serializers.CustomFieldSerializer

    valid_content_type_filter_fields = ["app_label", "model"]

    def get_queryset(self) -> QuerySet[models.CustomField]:
        qs = super().get_queryset()
        for field in self.valid_content_type_filter_fields:
            value = self.request.GET.get(field)
            if value:
                qs = qs.filter(**{f"content_type__{field}": value.lower()})
        return qs
